#!/usr/bin/env python
# Copyright (c) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2020 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENSE.TXT
"""This script helps install ELS repository for Ubuntu 18.04.

Before use the scirpt you should get a license key in CLN. Then use to to run
    the script and wait until it done.

You can find a new repository like this in your system:

deb https://repo.cloudlinux.com/ubuntu18_04-els/{}/updates bionic main
# deb-src https://repo.cloudlinux.com/ubuntu18_04-els/{}/updates bionic main


"""
from __future__ import print_function

import getopt
import json
import os
import re
import socket
import sys
from subprocess import PIPE, Popen

import ConfigParser
import urllib2

__VERSION__ = "0.5"
__REPO_FILE__ = "ubuntu-els.list"
__CLN_SERVER__ = "https://cln.cloudlinux.com"
__TOKEN_END_POINT__ = "/cln/api/centos/token/register"
__GPG_KEY_URL__ = "https://www.repo.cloudlinux.com/cloudlinux/security/RPM-GPG-KEY-CloudLinux"


def get_hostname():
    """Get local hostname."""
    return socket.gethostname()


def show_help():
    """Show help message."""
    print(
        """Install script for Ubuntu 18.04 Extended Life Support by Cloudlinux repository

Options:
    -h, --help      Show this help message
    -V, --version   Show version information
    --license-key   User license key
    --cln-server    CLN server
    --hostname      Custom hostname
    --force         Force re-register if repo exist

Usage:
    ./convert_rhel_to_centos_els.py --license-key <KEY>
"""
    )


def do_opts():
    """Parse of command line arguments."""
    if len(sys.argv) == 1:
        show_help()
        sys.exit(1)

    optlist, args = getopt.getopt(
        sys.argv[1:],
        "hVf",
        ["help", "version", "force", "license-key=", "cln-server=", "hostname="],
    )

    params = dict(optlist)

    if "-h" in params or "--help" in params:
        show_help()
        sys.exit(1)

    ret = {
        "show_version": False,
        "force": False,
        "cln_server": __CLN_SERVER__,
        "hostname": get_hostname(),
    }

    if "--version" in params or "-V" in params:
        ret["show_version"] = True

    if "--force" in params or "-f" in params:
        ret["force"] = True

    if "--license-key" in params:
        ret["license_key"] = params["--license-key"]
    else:
        print("Forgot required argument: --license-key")
        show_help()
        sys.exit(1)

    if "--cln-server" in params:
        ret["cln_server"] = params["--cln-server"]

    if "--hostname" in params:
        ret["hostname"] = params["--hostname"]

    return ret


def repo_path(repo_file):
    """Return path to repo file."""
    return os.path.join("/etc/apt/sources.list.d/", repo_file)


def pre_run_check():
    """Check that repo files exist."""
    path = repo_path(__REPO_FILE__)

    print("Check that repository isn't created... ", end="")
    if os.path.isfile(path):
        print("Repository exists.")
        return False

    print("Ok")
    return True


def show_version():
    """Show script version."""
    print(__VERSION__)


def make_request_url(server, endpoint):
    """Join CLN host and endpoint."""
    return os.path.join(server.rstrip("/"), endpoint.lstrip("/"))


def get_auth_token(url, hostname, lic):
    """Get auth token for repo config."""
    print("Request repository token for this server... ", end="")
    data = json.dumps({"key": "{0}".format(lic), "host_name": "{0}".format(hostname)})
    req = urllib2.Request(
        url, data, {"content-type": "application/json", "accept": "*/*"}
    )
    try:
        ret = urllib2.urlopen(req)
    except urllib2.HTTPError as ex:
        if 400 <= ex.code < 500:
            data = json.loads(ex.read())
            msg = data.get('message')
            details = data.get('details', 'no details')
            print("\n{0}\n{1}".format(msg, details))
        else:
            print("Error ({0})".format(ex))
        return False

    if ret.code > 200:
        print("Error (HTTP Error not 200 is {0})".format(ret.code))

    data = json.loads(ret.read())

    if "token" in data:
        print("Ok")
        return data["token"]

    print("Error (CLN returns no token")
    return False


def get_repo_config(token):
    """Generate content for repo file."""
    repo = """
deb [arch=amd64] https://repo.cloudlinux.com/ubuntu18_04-els/{}/updates bionic main
# deb-src https://repo.cloudlinux.com/ubuntu18_04-els/{}/updates bionic main    
""".format(token, token)
    return repo


def save_repo(repo, repo_file):
    """Save repo data into yum.repos.d folder."""
    path = repo_path(repo_file)
    print("Save repo file to {0}... ".format(path), end="")

    try:
        with open(path, "w") as f:
            f.write(repo)
        print("Ok")
        return True
    except IOError as ex:
        print("I/O error({0}): {1}".format(ex.errno, ex.strerror))
        return False


def run_shell_command(cmd, err_msg="Unknown problem."):
    """Run shell command, and show additional information in case error."""
    process = Popen(" ".join(cmd), shell=True, stdout=PIPE, stderr=PIPE)
    process.wait()

    if process.returncode > 0:
        message = """

        {0}
        Command: {1}
        Return code: {2}
        STDOUT:
        ===============
        {3}
        ===============
        STDERR:
        ===============
        {4}
        ===============

        """
        print(
            message.format(
                err_msg,
                " ".join(cmd),
                process.returncode,
                process.stdout.read(),
                process.stderr.read(),
            )
        )
        return False

    return True


def update_apt_cache():
    """Clean rpm cache for all packages."""
    print("Cleaning repository cache... ", end="")
    cmd = ["apt", "update"]

    result = run_shell_command(
        cmd, err_msg="Problem with updating apt cache."
    )

    if result:
        print("Ok")

    return result


def ask_about_collecting_data():
    """Ask user about collecting data."""
    print(
        """Hello.
We want collect data about installed packages.
This need for make our support more usefull for you.
Do you want help us and give grant for collect data?"""
    )
    answer = raw_input("(y/[n])")
    if answer.lower() in ("", "n", "no"):
        return False
    else:
        return True


def collect_data():
    """Collect data about installed packages."""
    pass


def get_dist_name():
    """Get dist name from /etc/system-release."""
    system_release_path = "/etc/lsb-release"
    dist_name = "not-ubuntu"

    print("Get dist name... ", end="")
    if not os.path.isfile(system_release_path):
        print("It's not Debian based distr.")
        return dist_name

    with open(system_release_path, "r") as system_release_file:
        system_release_data = system_release_file.read()
        dist_name_pattern = re.compile(r"^DISTRIB_ID=(.*)")
        dist_name_matches = dist_name_pattern.findall(system_release_data)
        if len(dist_name_matches) == 0:
            print("It's not Debian based distr")
            return dist_name
        
        dist_name = dist_name_matches[0]

    print(dist_name)

    return dist_name


def verify_token(token):
    """Verify token."""
    pass


def add_gpg_key_to_apt():
    """Add Cloudlinux gpg key to apt"""
    print("Add Cloudlinux gpg key to apt... ", end="")
    cmd = ["apt", "install", "gnupg2", "-y"]
    
    result = run_shell_command(
        cmd, err_msg="Problem with installing gnupg2."
    )
    
    if not result:
        return result

    cmd = ["apt-key", "adv", "--fetch-keys", __GPG_KEY_URL__]

    result = run_shell_command(
        cmd, err_msg="Problem with adding gpg key."
    )

    if result:
        print("Ok")

    return result

def remove_els_repo():
    """Remove els repo."""
    path = repo_path(__REPO_FILE__)
    print("Removing repo file {0}... ".format(path), end="")

    try:
        os.remove(path)
        print("Ok")
    except OSError as ex:
        print("OS error({0}): {1}".format(ex.errno, ex.strerror))


def main():
    """Read arguments, get server token and create a new repo."""
    args = do_opts()
    cln_server = args["cln_server"]
    hostname = args["hostname"]
    lic = args["license_key"]
    force = args["force"]

    if args["show_version"]:
        show_version()
        return 0

    dist_name = get_dist_name()

    if dist_name == "not-ubuntu":
        return 11

    if not pre_run_check():
        if force:
            remove_els_repo()
        else:
            print("For re-registration license run script with --force .")
            return 2

    url = make_request_url(cln_server, __TOKEN_END_POINT__)
    print(url)
    token = get_auth_token(url, hostname, lic)
    if not token:
        return 3

    repo = get_repo_config(token)
    if not save_repo(repo, __REPO_FILE__):
        return 4

    if not add_gpg_key_to_apt():
        return 7

    if not update_apt_cache():
       return 6

    return 0


if __name__ == "__main__":
    sys.exit(main())

